kura_types (kura v1.19.2)
View SourceType system for casting, dumping, and loading values between Erlang and PostgreSQL.
Supported types: id, integer, float, string, text, boolean,
date, utc_datetime, uuid, jsonb, {array, Type}.
Summary
Functions
Coerce external input to an Erlang term for the given type.
Convert an Erlang term to a pgo-compatible value for storage.
Convert a pgo result value back to an Erlang term.
Return the PostgreSQL DDL type string for a kura type.
Types
Functions
Coerce external input to an Erlang term for the given type.
Convert an Erlang term to a pgo-compatible value for storage.
Convert a pgo result value back to an Erlang term.
Return the PostgreSQL DDL type string for a kura type.